import sys
sys.path.append( '../..' )
from src.user.database.database import MongoClient
import hashlib
import maskpass

class User(object):
	"""docstring for User"""
	def __init__(self):
		super(User, self).__init__()
		self.username = ""
		self.logged_in = ""
		self.email = ""
		self.mongo_client = MongoClient()
		self.user_db = self.mongo_client.getDatabase('user_db')
		self.user_col = self.mongo_client.getCollection('user_data',self.user_db)

	def loginUser(self):
		username = input("Enter username: ")
		password = maskpass.askpass(prompt="Enter password: ", mask="#")

		user_id = self.mongo_client.findInCollection(self.user_col,{'username' : username})

		# should be only once
		l = len(user_id)
		for x in user_id:
			ref = x['password']
			email = x['email']

		if l == 0:
			print('wrong username or password')
			return self.loginUser()

		hashed_password = hashlib.sha256(password.encode('utf-8')).hexdigest()

		if hashed_password == ref:
			self.username = username
			self.email = email
			return True
		else:
			print('wrong username or password')
			return self.loginUser()
		return self.loginUser()

	def createUser(self):
		# reads from prompt
		username = input("Enter username: ")
		password = maskpass.askpass(prompt="Enter password: ", mask="#")
		password2 = maskpass.askpass(prompt="Enter password again: ", mask="#")

		email = input("Enter your e-mail: ")
		email2 = input("Enter your e-mail again: ")

		if not email == email2:
			print('emails do not match...')
			print('Try again')
			return self.createUser()

		if not password == password2:
			print('passwords do not match...')
			print('Try again')
			return self.createUser()

		# checks user does not exist yet
		user_id = self.mongo_client.findInCollection(self.user_col,{'username' : username})

		for x in user_id:
			if x['username'] == username:
				print('Username taken!')
				return self.createUser()

		# encrypts password
		hashed_password = hashlib.sha256(password.encode('utf-8')).hexdigest()

		# stored user data
		self.mongo_client.insert2Collection({'username' : username,'email' : email, 'password' : hashed_password},self.user_col)
		self.username = username
		self.email = email
		return True

	def deleteUser(self):
		# reads from prompt
		username = input("Enter username: ")

		# checks user does not exist yet
		user_id = self.mongo_client.findInCollection(self.user_col,{'username' : username})

		l = 0
		for x in user_id:
			l = l + 1
			ref = x['password']

		if l == 0:
			ref = ''

		password = maskpass.askpass(prompt="Enter password: ", mask="#")
		password2 = maskpass.askpass(prompt="Enter password again: ", mask="#")

		if not password == password2:
			print('passwords do not match...')
			print('Operation aborted')
			return False

		hashed_password = hashlib.sha256(password.encode('utf-8')).hexdigest()

		if hashed_password == ref:
			print("Are you sure you want to proceed? all data will be erased")
			yn = input("[Y/n]: ")

			if yn == 'y' or yn == 'Y':
				# stored user data
				self.mongo_client.deleteFromCollection({'username' : username},self.user_col)

			return True
		else:
			print('wrong username or password')
			print('Operation aborted')
			return False

def main():
	usr = User()
	usr.createUser()
	usr.loginUser()
	usr.deleteUser()

if __name__ == '__main__':
	main()



import sys
sys.path.append( '../../../' )
import pymongo
import urllib 
from dotenv import *

config = dotenv_values()

class MongoClient(object):
	"""docstring for Database"""
	def __init__(self):
		super(MongoClient, self).__init__()
		uri = "mongodb+srv://baceituno:"+urllib.parse.quote(config['MONGODB_API_KEY'])+"@cluster0.utuloy2.mongodb.net/?retryWrites=true&w=majority"
		self.client = pymongo.MongoClient(uri)
		self.client.test

	def getDatabase(self, db_name):
		return self.client[db_name]

	def getCollection(self, collection, db):
		return db[collection]

	def insert2Collection(self, item, collection):
		return collection.insert_one(item)

	def deleteFromCollection(self, item, collection):
		return collection.delete_one(item)
		
	def insert_many2Collection(self, items, collection):
		return collection.insert_many(items)
		
	def findInCollection(self, collection, query):
		return collection.find(query)

if __name__ == '__main__':
	client = MongoClient()

	db = client.getDatabase('db0')
	collection = client.getCollection('col0',db)
	res = client.insert2Collection({'a' : 'b'},collection)
	res = client.findInCollection(collection,{'a':'b'})



import sys
sys.path.append( '../../..' )
from datetime import *
from tzlocal import get_localzone
import cv2
import numpy as np
import time
import os
import copy
import io
import json
from pathlib import Path
from src.comm.docker_ver import *
path_home = os.getenv('LCP_DKR')+'/' if docker_ver() else str(Path.home())

class spacy_ner_schema(object):
	def __init__(self, init):
		self.init = init
		self.schema = None
		self.metadata = None
		self.schema_path = self.init.prefix_meta + 'schema.json'
		self.meta_path = self.init.prefix_meta + 'spacy_ner_data.json'
		self.status = {}
		self.filtered = False
		self.sliced = False
		self.selected_slices = False
		self.bounding_box_thumbs = True
		
		self.in_version = False
		self.version_keys = None
		self.selected_version = None
		self.version_schema = ''

		ls, _ = self.init.storage.load_dataset_list()
		self.ls = ls

	def create_schema_file(self):
		# generates the schema file for a yolo dataset
		schema = {}
		current = self.init.load_current()
		k = 0
		idx = 0

		# finds the images
		for key in current['keys']:
			if '.json' in key:
				idx_key = 0
				local_array = json.load(self.init.storage.load_file_global(key))
				for el in local_array:
					dp = {}
					labels = el['entities']
					dp['filename'] = key
					dp['key'] = el['text']
					dp['text'] = el['text']
					dp['lm'] = current['lm'][idx]
					dp['entities'] = labels
					dp['entity_types'] = [label['type'] for label in labels]
					dp['n_entities'] = len(dp['entities'])
					dp['tags'] = []
					dp['slices'] = []
					dp['length'] = len(dp['text'])
					dp['idx'] = idx_key
					schema[el['text']] = dp
					idx_key +=1
					k += 1
			idx += 1

		schema['len'] = k

		# stores dp
		self.schema = schema
		self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(schema).encode('ascii')))
		self.init.storage.reset_buffer()
		self.copy_schema_to_latest_version_checkpoint()

		return True

	def get_labels(self, filename, version='current'):
		schema = self.get_schema()
		return schema[filename]['entities']
	
	def copy_schema_to_latest_version_checkpoint(self):
		try:
			versions = self.init.load_versions()
		except:
			self.init.setup_versions()
			versions = self.init.load_versions()

		schema_copy_path = self.init.prefix_versions + '/schemas/' + f'{len(versions.keys())}.json' 
		schema_copy = copy.deepcopy(self.schema)

		for dp in schema_copy:
			if type(schema_copy[dp]) is dict:
				ver = self.init.get_latest_diff_number(schema_copy[dp]['key'])
				schema_copy[dp]['key'] = self.init.prefix_diffs + schema_copy[dp]['key'] + '/' + str(ver).zfill(10)

		self.init.storage.add_file_from_binary_global(schema_copy_path,io.BytesIO(json.dumps(schema_copy).encode('ascii')))
		keys = list(versions.keys())
		versions[keys[-1]]['schema_path'] = schema_copy_path

		metapath = self.init.prefix_versions + 'versions.json'
		self.init.storage.add_file_from_binary_global(metapath,io.BytesIO(json.dumps(versions).encode('ascii')))
		return True

	def select_version(self, version, schema_file, keys):
		self.in_version = True
		self.selected_version = version
		self.version_schema = schema_file
		self.ls = keys.values()

	def reset_to_current_version(self):
		self.in_version = False
		self.version_keys = None
		self.selected_version = None
		self.schema = None
		self.filtered = None
		self.sliced = None

		ls, _ = self.init.storage.load_dataset_list()
		self.ls = ls
		self.schema = json.load(self.init.storage.load_file_global(self.schema_path))

		return True

	def get_schema(self):
		if self.schema == None:
			try:
				if self.in_version:
					self.schema = json.load(self.init.storage.load_file_global(self.version_schema))
				else:
					self.schema = json.load(self.init.storage.load_file_global(self.schema_path))
			except:
				self.create_schema_file()
		return self.schema

	def compute_meta_data(self):
		schema = self.get_schema()
		
		entities = []
		lengths = []
		lm = []
		tags = []
		entities_per_sentence = []
		slices = []

		n_entity = {}
		n_len = {}
		n_lm = {}
		n_tags = {}
		n_slices = {}

		for val in schema:
			if type(self.schema[val]) is dict:
				if not len(schema[val]['entities']) in entities_per_sentence:
					entities_per_sentence.append(len(schema[val]['entities']))
				
				if 'slice' in schema[val].keys():
					for sl in schema[val]['slices']:
						if not sl in slices:
							slices.append(sl)
							n_slices[sl] = 1
						else:
							n_slices[sl] += 1

				for cl in schema[val]['entities']:
					if not cl['type'] in entities:
						entities.append(cl['type'])
						n_entity[cl['type']] = 1
					else:
						n_entity[cl['type']] += 1
				
				if not schema[val]['length'] in lengths:
					lengths.append(schema[val]['length'])
					n_len[schema[val]['length']] = 1
				else:
					n_len[schema[val]['length']] += 1
				
				if not schema[val]['lm'] in lm:
					lm.append(schema[val]['lm'])
					n_lm[schema[val]['lm']] = 1
				else:
					n_lm[schema[val]['lm']] += 1

				if 'tags' in schema[val].keys():
					for tag in schema[val]['tags']:
						if not tag in tags:
							tags.append(tag)
							n_tags[tag] = 1
						else:
							n_tags[tag] += 1

		metadata = {'entities': entities, 'lengths': lengths, 'lm': lm, 'slices': slices, 'n_slices': n_slices, 'tags': tags, 'n_entity': n_entity, 'n_len': n_len, 'n_lm': n_lm, 'n_tags': n_tags, 'entities_per_sentence': entities_per_sentence}
		self.metadata = metadata

		if not self.in_version:
			self.init.storage.add_file_from_binary_global(self.meta_path,io.BytesIO(json.dumps(metadata).encode('ascii')))
			self.init.storage.reset_buffer()

		return True

	def update_schema_file(self,added=[],modified=[],removed=[]):
		# loads the existing schema file
		schema = self.get_schema()
		# finds the images

		print('updating schema file')
		
		idx = int(schema['len'])

		for key in removed:
			for el in schema:
				if schema[el]['filename'] == key:
					del schema[el]
					idx -= 1

		for key in added:
			if '.json' in key:
				idx_key = 0
				local_array = json.load(self.init.storage.load_file_global(key))
				for el in local_array:
					dp = {}
					labels = el['entities']
					dp['filename'] = key
					dp['key'] = el['text']
					dp['text'] = el['text']
					dp['lm'] = self.init.storage.load_file_metadata_global(key)['last_modified']
					dp['entities'] = labels
					dp['entity_types'] = [label['type'] for label in labels]
					dp['n_entities'] = len(dp['entities'])
					dp['tags'] = []
					dp['slices'] = []
					dp['length'] = len(dp['text'])
					dp['idx'] = idx_key
					schema[el['text']] = dp
					idx_key +=1	
			idx += 1

		keys = set(schema.keys())

		for key in modified:
			if '.json' in key:
				idx_key = 0
				local_array = json.load(self.init.storage.load_file_global(key))
				for el in local_array:
					if el['text'] in list(keys):
						dp = {}
						labels = el['entities']
						dp['filename'] = key
						dp['key'] = el['text']
						dp['text'] = el['text']
						print(labels)
						print(el['entities'])
						if labels == schema[el['text']]['entities']:
							dp['lm'] = schema[el['text']]['lm']	
						else:
							dp['lm'] = self.init.storage.load_file_metadata_global(key)['last_modified']
						dp['entities'] = labels
						dp['entity_types'] = [label['type'] for label in labels]
						dp['n_entities'] = len(dp['entities'])
						dp['tags'] = []
						dp['slices'] = []
						dp['length'] = len(dp['text'])
						dp['idx'] = idx_key
						schema[el['text']] = dp
					else:
						dp = {}
						labels = el['entities']
						dp['filename'] = key
						dp['key'] = el['text']
						dp['text'] = el['text']
						dp['lm'] = self.init.storage.load_file_metadata_global(key)['last_modified']
						dp['entities'] = labels
						dp['entity_types'] = [label['type'] for label in labels]
						dp['n_entities'] = len(dp['entities'])
						dp['tags'] = []
						dp['slices'] = []
						dp['length'] = len(dp['text'])
						dp['idx'] = idx_key
						schema[el['text']] = dp
						idx += 1
					idx_key +=1

				local_keys = set([el['text'] for el in local_array])
				keys_new = set(schema.keys())
				for dp in list(keys_new):
					if not dp in local_keys:
						if type(schema[dp]) is dict:
							if schema[dp]['filename'] == key:
								del schema[dp]
								idx -= 1

		schema['len'] = idx

		# stores dp
		self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(schema).encode('ascii')))
		self.init.storage.reset_buffer()
		self.schema = schema

		return self.compute_meta_data()
	
	def get_tags(self, key):
		keys = self.schema.keys()
		for val in keys:
			if type(self.schema[val]) is dict:
				if key in self.schema[val]['key']:
					if 'tags' in self.schema[val].keys():
						return self.schema[val]['tags']
		return []

	def add_tag(self, key, tag):
		for val in self.schema.keys():
			if type(self.schema[val]) is dict:
				if key == self.schema[val]['key']:
					if 'tags' in self.schema[val].keys():
						if not tag in self.schema[val]['tags']:
							self.schema[val]['tags'].append(tag)
					else:
						self.schema[val]['tags'] = []
						self.schema[val]['tags'].append(tag)

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def add_many_tag(self, keys, tag):
		for val in self.schema.keys():
			if type(self.schema[val]) is dict:
				if self.schema[val]['key'] in keys:
					if 'tags' in self.schema[val].keys():
						if not tag in self.schema[val]['tags']:
							self.schema[val]['tags'].append(tag)
					else:
						self.schema[val]['tags'] = []
						self.schema[val]['tags'].append(tag)
					keys.remove(self.schema[val]['key'])

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def remove_tag(self, key, tag):
		keys = self.schema.keys()
		for val in keys:
			if type(self.schema[val]) is dict:
				if key in self.schema[val]['key']:
					if 'tags' in self.schema[val].keys():
						if tag in self.schema[val]['tags']:
							self.schema[val]['tags'].remove(tag)

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def remove_all_tags(self, key):
		keys = self.schema.keys()
		for val in keys:
			if type(self.schema[val]) is dict:
				if key in self.schema[val]['key']:
					if 'tags' in self.schema[val].keys():
						self.schema[val]['tags'] = []

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def many_remove_all_tags(self, keys):
		for val in self.schema.keys():
			if type(self.schema[val]) is dict:
				if self.schema[val]['key'] in keys:
					if 'tags' in self.schema[val].keys():
						self.schema[val]['tags'] = []
					keys.remove(self.schema[val]['key'])

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def label_name(self, class_number):
		return str(class_number)

	def set_labels(self, sentence, labels_array):
		schema = self.get_schema()
		dp = schema[sentence]
		local_array = json.load(self.init.storage.load_file_global(dp['filename']))
		for idx in range(len(local_array)):
			if local_array[idx]['text'] == sentence:
				local_array[idx]['entities'] = labels_array
		self.init.storage.add_file_from_binary_global(dp['filename'],io.BytesIO(json.dumps(local_array).encode('ascii')))
		return True
	
	def branch(self, branch_name, type_ ='copy'):
		schema = self.get_schema()
		dataset = self.init.storage.dataset
		if self.init.storage.type == 'local':
			branch_name = path_home + '/' + branch_name + '/'
		files = {}

		if self.filtered:
			status = self.status
		else:
			status = self.read_all_files()

		for dp in status['keys']:
			if schema[dp]['filename'] in files.keys():
				files[schema[dp]['filename']].extend(dp)
			else:
				files[schema[dp]['filename']] = [dp]

		for key in files.keys():
			local_array = [{'text': schema[dp]['text'], 'entities': schema[dp]['entities']} for dp in files[key]]
			self.init.storage.add_file_from_binary_global(branch_name + key.replace(dataset,''),io.BytesIO(json.dumps(local_array).encode('ascii')))
		
		return True

	def merge(self, goal):
		dataset = self.init.storage.dataset
		if self.init.storage.type == 'local':
			goal = goal.replace(path_home,'')
			goal = path_home + '/' + goal + '/'

		if self.filtered:
			status = self.status
		else:
			status = self.read_all_files()

		schema = self.get_schema()
		files = {}
		for dp in status['keys']:
			if schema[dp]['filename'] in files.keys():
				files[schema[dp]['filename']].extend(dp)
			else:
				files[schema[dp]['filename']] = [dp]

		files_goal, _ = self.init.storage.load_list_in_path(goal)
		files_goal = [f.replace(goal,'') for f in files_goal]
		for file in files:
			if file.replace(dataset,'') in files_goal:
				local_array = json.load(self.init.storage.load_file_global(file))
				missing = [dp for dp in files[file]]
				for idx in range(len(local_array)):
					for dp in missing:
						if local_array[idx]['text'] == dp:
							local_array[idx]['entities'] = schema[dp]['entities']
							missing.pop(dp)
					if len(missing) == 0:
						break
				for dp in missing:
					local_array.extend({'text': schema[dp]['text'], 'entities': schema[dp]['entities']})
				self.init.storage.add_file_from_binary_global(goal+file.replace(dataset,''),io.BytesIO(json.dumps(local_array).encode('ascii')))
			else:
				self.init.storage.copy_file_global(file,goal+file.replace(dataset,''))
		return True

	def read_all_files(self):
		# queries the json
		schema = self.get_schema()
		status = {'keys': [], 'lm': [], 'filename': [], 'dp': []}
		for dp in schema:
			if type(self.schema[dp]) is dict:
				status['keys'].append(schema[dp]['key'])
				status['filename'].append(schema[dp]['filename'])
				status['lm'].append(schema[dp]['lm'])
				status['dp'].append(schema[dp]['entities'])
		self.status = status
		return status

	def get_metadata(self):
		if self.filtered or self.in_version:
			schema = self.get_schema()
		
			entities = []
			lengths = []
			lm = []
			tags = []
			entities_per_sentence = []
			slices = []

			n_entity = {}
			n_len = {}
			n_lm = {}
			n_tags = {}
			n_slices = {}

			for val in schema:
				if type(self.schema[val]) is dict:
					if not len(schema[val]['entities']) in entities_per_sentence:
						entities_per_sentence.append(len(schema[val]['entities']))
					
					if 'slice' in schema[val].keys():
						for sl in schema[val]['slices']:
							if not sl in slices:
								slices.append(sl)
								n_slices[sl] = 1
							else:
								n_slices[sl] += 1

					for cl in schema[val]['entities']:
						if not cl['type'] in entities:
							entities.append(cl['type'])
							n_entity[cl['type']] = 1
						else:
							n_entity[cl['type']] += 1
					
					if not schema[val]['length'] in lengths:
						lengths.append(schema[val]['length'])
						n_len[schema[val]['length']] = 1
					else:
						n_len[schema[val]['length']] += 1
					
					if not schema[val]['lm'] in lm:
						lm.append(schema[val]['lm'])
						n_lm[schema[val]['lm']] = 1
					else:
						n_lm[schema[val]['lm']] += 1

					if 'tags' in schema[val].keys():
						for tag in schema[val]['tags']:
							if not tag in tags:
								tags.append(tag)
								n_tags[tag] = 1
							else:
								n_tags[tag] += 1

			return {'entities': entities, 'lengths': lengths, 'lm': lm, 'slices': slices, 'n_slices': n_slices, 'tags': tags, 'n_entity': n_entity, 'n_len': n_len, 'n_lm': n_lm, 'n_tags': n_tags, 'entities_per_sentence': entities_per_sentence}
		else:
			return json.load(self.init.storage.load_file_global(self.meta_path))

	def reset_filters(self):
		self.filtered = False
		return True
	
	def apply_filters(self, filters={}):
		schema = self.get_schema()
		status = {'keys': [], 'lm': [], 'filename': [], 'dp': []}

		if len(filters) == 0:
			self.filtered = False
			return self.status

		for dp in schema:
			if type(self.schema[dp]) is dict:
				if self.sliced:
					if 'slices' in schema[dp].keys():
						pre_add = any([sl in schema[dp]['slices'] for sl in self.selected_slices])
					else:
						pre_add = False
				else:
					pre_add = True

				if pre_add:
					add_num =  []
					add_entity =  []
					add_len =  []
					add_name =  []
					add_tag =  []
					add_date =  []
					add_box =  []

					for f in filters:
						for filt in filters[f]:
							if filt == 'num_entities':
								
								min_cl = int(filters[f]['num_entities'][0])
								max_cl = int(filters[f]['num_entities'][1])
								
								if (len(schema[dp]['entities']) <= max_cl) and (len(schema[dp]['entities']) >= min_cl):
									add_num.append(True)
								else:
									add_num.append(False)
							
							if filt == 'entity':
								if filters[f]['entity'] in schema[dp]['entity_types']:
									add_entity.append(True)
								else:
									add_entity.append(False)
									
							if filt == 'length':
								if int(filters[f]['length']) == schema[dp]['length']:
									add_len.append(True)
								else:
									add_len.append(False)

							if filt == 'name':
								if filters[f]['name'] in schema[dp]['text']:
									add_name.append(True)
								else:
									add_name.append(False)

							if filt == 'tag':
								if 'tags' in schema[dp]:
									if filters[f]['tag'] in schema[dp]['tags']:
										add_tag.append(True)
									else:
										add_tag.append(False)
								else:
									add_tag.append(False)

							if filt == 'date':
								d_min = datetime.strptime(filters[f]['date'][0], '%Y/%m-%d').date()
								d_max = datetime.strptime(filters[f]['date'][1], '%Y/%m-%d').date()
								date = datetime.strptime(schema[dp]['lm'], '%m/%d/%Y, %H:%M:%S').astimezone(get_localzone()).date()

								if date <= d_max and date >= d_min:
									add_date.append(True)
								else:
									add_date.append(False)

							if filt == 'entity_len':
								
								a_min = min(filters[f]['entity_len'][0], filters[f]['entity_len'][1])
								a_max = max(filters[f]['entity_len'][0], filters[f]['entity_len'][1])

								
								if 'entities' in schema[dp]:
									if len(schema[dp]['entities']) == 0:
										add_box.append(False)

									for label in schema[dp]['entities']:
										area = label['end'] - label['start'] + 1
										
										if (area >= a_min) and (area <= a_max):
											add_box.append(True)
										else:
											add_box.append(False)
								else:
									add_box.append(False)
					
					if len(add_entity) == 0:
						add_entity = [True]
					if len(add_len) == 0:
						add_len = [True]
					if len(add_name) == 0:
						add_name = [True]
					if len(add_tag) == 0:
						add_tag = [True]
					if len(add_box) == 0:
						add_box = [True]
					if len(add_num) == 0:
						add_num = [True]
					if len(add_date) == 0:
						add_date = [True]
						
					add = all([any(add_entity),any(add_len),any(add_name),any(add_tag),any(add_box),any(add_num),any(add_date)])
					if add:
						status['keys'].append(schema[dp]['key'])
						status['filename'].append(schema[dp]['filename'])
						status['lm'].append(schema[dp]['lm'])
						status['dp'].append(schema[dp]['entities'])

		self.filtered = True
		self.status = status
		return status

	def add_slice(self, slice_name=''):
		status = set(self.status['keys'])
		for val in self.schema.keys():
			if type(self.schema[val]) is dict:
				if self.schema[val]['key'] in status:
					if 'slices' in self.schema[val].keys():
						if not slice_name in self.schema[val]['slices']:
							self.schema[val]['slices'].append(slice_name)
					else:
						self.schema[val]['slices'] = [slice_name]

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def remove_slice(self, slice_name=''):
		for val in self.schema.keys():
			if type(self.schema[val]) is dict:
				if 'slices' in self.schema[val].keys():
					if slice_name in self.schema[val]['slices']:
						self.schema[val]['slices'].remove(slice_name)

		# stores schema file
		if self.in_version:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.version_schema).encode('ascii')))
		else:
			self.init.storage.add_file_from_binary_global(self.schema_path,io.BytesIO(json.dumps(self.schema).encode('ascii')))
		self.init.storage.reset_buffer()

		return self.compute_meta_data()

	def get_slices(self):
		schema = self.get_schema()
		n_slices = {}
		slices = []
		for val in schema:
			if type(self.schema[val]) is dict:
				if 'slices' in schema[val].keys():
						for sl in schema[val]['slices']:
							if not sl in slices:
								slices.append(sl)
								n_slices[sl] = 1
							else:
								n_slices[sl] += 1
		return n_slices

	def get_slices_key(self, key):
		keys = self.schema.keys()
		for val in keys:
			if type(self.schema[val]) is dict:
				if key in self.schema[val]['key']:
					if 'slices' in self.schema[val].keys():
						return self.schema[val]['slices']
		return []
		
	def select_slice(self, slices=[]):
		if len(slices) == 0:
			self.sliced = False
			self.selected_slices = []
			return self.status
		self.sliced = True
		self.selected_slices = slices
		return self.apply_filters({'slic': []})
	
	def get_status(self):
		return self.status

	def get_dp(self, sentence):
		schema = self.get_schema()
		return schema[sentence]	


# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['src',
 'src.comm',
 'src.core',
 'src.dataset',
 'src.dataset.named_entity_recognition',
 'src.dataset.object_detection',
 'src.storage',
 'src.storage.classes',
 'src.training',
 'src.user',
 'src.user.database',
 'src.user.license',
 'stack_api']

package_data = \
{'': ['*']}

install_requires = \
['boto3>=1.24.60,<2.0.0',
 'csv-diff',
 'fastapi>=0.80.0',
 'google-cloud-datastore>=2.8.1,<3.0.0',
 'google-cloud-storage>=2.5.0,<3.0.0',
 'maskpass>=0.3.6,<0.4.0',
 'opencv-python>=4.6.0.66',
 'pandas',
 'pymongo>=4.2.0,<5.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'typer[all]>=0.6.1,<0.7.0',
 'tzlocal']

entry_points = \
{'console_scripts': ['stack = stack_api.api_cli:app']}

setup_kwargs = {
    'name': 'stackcli',
    'version': '0.1.22',
    'description': '',
    'long_description': "# Stack\n\n### Install Stack\n\n1. Open your favorite terminal: `Terminal`\n2. Install stackcli: `pip install stackcli` (ideally in a virtualenv)\n\n### Build a test dataset\n\n1. Make a directory: `mkdir test_dataset`\n3. Download an image of Einstein: `curl -o einstein.jpg https://upload.wikimedia.org/wikipedia/en/8/86/Einstein_tongue.jpg`\n\n\n### Try Stack's Command Line Tool (CLI)\n\n1. Init stack in the current directory: ```stack init ./test_dataset ``` (note the dot at the beginning)\n2. Add a file to track: `stack add einstein.jpg`\n3. Commit your changes: `stack commit`\n4. Check status: `stack status`\n5. See history of changes: `stack history`\n6. Remove the file: `stack remove einstein.jpg`\n7. Revert your changes: `stack revert 1`\n8. You should see Einstein in your directory again!\n\n\n# Stack Dev (Ignore if you are just a user)\n\n### Features to implement\n\n### Publish to TestPy\n\nhttps://typer.tiangolo.com/tutorial/package/\n\n1. Install package: `poetry install'\n2. Try CLI: find it first `which stack`\n3. Create a wheel package: `poetry build'\n4. Test wheel: `pip install .' # WARNING: Make sure you are in a virtualenv.\n5. Try the wheel: `stack`\n6. Publish it to TestPy: `poetry publish --build'\n(If you see a `HTTP Error 400: File already exists.` update version number in the `pyproject.toml` file)\n7. Install from TestPyPI:\n  1. `pip uninstall stack-cli`\n  1. `pip install -i https://test.pypi.org/simple/ stack-cli`\n",
    'author': 'Toni Rosinol',
    'author_email': 'arosinol@stack-ai.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}


setup(**setup_kwargs)



# from src.storage import Storage
import os
import boto3
import boto3.s3.transfer as s3transfer
import botocore
import configparser
import maskpass
import io
from pathlib import Path
import sys
sys.path.append( '../../../' )
from src.comm.docker_ver import *
path_home = os.getenv('LCP_DKR')+'/' if docker_ver() else str(Path.home())

class S3Bucket(object):
	"""docstring for Storage"""
	def __init__(self, BUCKET_NAME):
		# self = Cloud().__init__()
		self.type = "s3"
		self.dataset = ""
		self.BUCKET_NAME = BUCKET_NAME
		self.credentials = {
			"aws_access_key_id": "",
			"aws_secret_access_key": "",
			"region": "us-east-1",
		}
		self.resource = None
		self.s3t = None
		self.bucket = None

	def connect_bucket(self, verbose=False):
		print('connecting to bucket')
		# checks if the credentials are in the computer
		if not os.path.isfile(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials'):
			print("AWS is not setup in this machine. Please follow the following setups:")
			print('\n')
			print("1) Go to https://console.aws.amazon.com/iamv2/")
			print("2) Choose 'Users' and click on 'Add user'.")
			print("3) Give the user a name (for example, StackBot).")
			print("4) Check 'Enable programmatic access'.")
			print("5) go to 'Set permissions', click 'Attach Existing Policies'.")
			print("6) Check 'AmazonS3FullAccess'")
			print('\n')
			print('Please copy your keys...')
			print('\n')

			self.credentials['aws_access_key_id'] = maskpass.askpass(prompt="Access key ID: ", mask="#")
			self.credentials['aws_secret_access_key'] = maskpass.askpass(prompt="Secret access key: ", mask="#")
			print('\n')

			config = configparser.ConfigParser()
			config['default'] = {}
			config['default']['aws_access_key_id'] = self.credentials['aws_access_key_id']
			config['default']['aws_secret_access_key'] = self.credentials['aws_secret_access_key']
			os.mkdir(path_home+'/.aws/'+self.BUCKET_NAME+'/')
			with open(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials', 'w') as configfile:
				config.write(configfile)
		else:
			config = configparser.ConfigParser()
			config.read(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials')

			self.credentials['aws_access_key_id'] = config['default']['aws_access_key_id']
			self.credentials['aws_secret_access_key'] = config['default']['aws_secret_access_key']

		# checks if the config is there
		if not os.path.isfile(path_home+'/.aws/'+self.BUCKET_NAME+'/config'):
			print('Please select your region (https://docs.aws.amazon.com/general/latest/gr/rande.html#s3_region)')
			print('[Press enter for us-east-2]')

			self.credentials['region'] = input("region: ")

			if self.credentials['region'] == "":
				self.credentials['region'] = 'us-east-2'

			config = configparser.ConfigParser()
			config['default'] = {}
			config['default']['region'] = self.credentials['region']

			with open(path_home+'/.aws/'+self.BUCKET_NAME+'/config', 'w') as configfile:
				config.write(configfile)
		else:
			config = configparser.ConfigParser()
			config.read(path_home+'/.aws/'+self.BUCKET_NAME+'/config')
			self.credentials['region'] = config['default']['region']

		if verbose:
			print('Connecting to your bucket...')
		self.resource = boto3.resource('s3',aws_access_key_id=self.credentials['aws_access_key_id'],aws_secret_access_key=self.credentials['aws_secret_access_key'])
		self.reset_buffer()

		# checks if the target bucket exist
		found = False
		buckets = self.resource.buckets.all()

		try:
			for bucket in buckets:
				if bucket.name == self.BUCKET_NAME:
					found = True
					break
		except:
			print('invalid credentials')
			print('removing config files')
			os.remove(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials')
			os.remove(path_home+'/.aws/'+self.BUCKET_NAME+'/config')
			os.rmdir(path_home+'/.aws/'+self.BUCKET_NAME+'/')

			print('try again...')

			return self.connect_bucket()

		# creates a new bucket or picks another bucket

		if found == False:
			if verbose:
				print('could not find your desired bucket')
				print('connection failed')
				raise Exception('Could not find your desired bucket')
		else:
			self.bucket = self.resource.Bucket(name=self.BUCKET_NAME)

		return found

	def connect_bucket_api(self,keys_dict):
		# checks if the credentials are in the computer

		if keys_dict['key1'] == 'NoKey' or keys_dict['key2'] == 'NoKey':
			config = configparser.ConfigParser()
			config.read(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials')
			keys_dict['key1'] = config['default']['aws_access_key_id']
			keys_dict['key2'] = config['default']['aws_secret_access_key']

		if keys_dict['key3'] == 'NoRegion':
			config = configparser.ConfigParser()
			config.read(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials')
			keys_dict['key3'] = config['default']['region']

		self.credentials['aws_access_key_id'] = keys_dict['key1']
		self.credentials['aws_secret_access_key'] = keys_dict['key2']
		self.credentials['region'] = keys_dict['key3']

		config = configparser.ConfigParser()
		config['default'] = {}
		config['default']['aws_access_key_id'] = self.credentials['aws_access_key_id']
		config['default']['aws_secret_access_key'] = self.credentials['aws_secret_access_key']
		config['default']['region'] = self.credentials['region']

		print('saving credentials')
		if not os.path.isfile(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials'):
			try:
				os.mkdir(path_home+'/.aws/')
			except:
				pass
			try: 
				os.mkdir(path_home+'/.aws/'+self.BUCKET_NAME+'/')
			except:
				pass

		with open(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials', 'w') as configfile:
			config.write(configfile)

		config = configparser.ConfigParser()
		config['default'] = {}
		config['default']['region'] = self.credentials['region']

		print('saving config')
		with open(path_home+'/.aws/'+self.BUCKET_NAME+'/config', 'w') as configfile:
			config.write(configfile)

		print('Connecting to your bucket...')
		self.resource = boto3.resource('s3',aws_access_key_id=self.credentials['aws_access_key_id'],aws_secret_access_key=self.credentials['aws_secret_access_key'])
		self.reset_buffer()

		# checks if the target bucket exist
		buckets = self.resource.buckets.all()

		for bucket in buckets:
			if bucket.name == self.BUCKET_NAME:
				found = True
				self.bucket = self.resource.Bucket(name=self.BUCKET_NAME)
				return True

		return False

	def reconnect_bucket_api(self):
		# checks if the credentials are in the computer
		config = configparser.ConfigParser()
		config.read(path_home+'/.aws/'+self.BUCKET_NAME+'/credentials')

		self.credentials['aws_access_key_id'] = config['default']['aws_access_key_id']
		self.credentials['aws_secret_access_key'] = config['default']['aws_secret_access_key']
		self.resource = boto3.resource('s3',aws_access_key_id=self.credentials['aws_access_key_id'],aws_secret_access_key=self.credentials['aws_secret_access_key'])
		self.reset_buffer()
		
		# checks if the target bucket exist
		buckets = self.resource.buckets.all()

		for bucket in buckets:
			if bucket.name == self.BUCKET_NAME:
				found = True
				self.bucket = self.resource.Bucket(name=self.BUCKET_NAME)
				return True

		return False

	def create_dataset(self,location):
		# assigns location
		if location[-1] != '/':
			location = location + '/'
		self.dataset = location
		self.raw_location = location
		return True

	def add_file(self,filepath,target_name='',subpath=''):
		# adding new file
		if target_name == '':
			target_name = os.path.basename(filepath)
		if filepath[-1] == '/':
			for root,dirs,files in os.walk(filepath):
				for file in files:
					self.s3t.upload(os.path.join(root,file),self.BUCKET_NAME,self.dataset+subpath+file)
			self.reset_buffer()
		else:
			self.s3t.upload(filepath,self.BUCKET_NAME,self.dataset+subpath+os.path.basename(filepath))
		return True

	def add_file_from_binary(self,filename,f):
		return self.add_file_from_binary_global(self.dataset+filename,f)

	def add_file_from_binary_global(self,filename,f):
		self.s3t.upload(f,self.BUCKET_NAME,filename)
		return True

	def remove_file(self,filename):
		# deletes files in path
		return self.remove_file_global(self.dataset+filename)

	def remove_file_global(self,filename):
		# deletes files in path
		if filename[-1] == '/':
			for obj in self.bucket.objects.filter(Prefix=filename):
				self.s3t.delete(self.BUCKET_NAME, obj.key)
			self.reset_buffer()
		elif filename[-1] == '*':
			for obj in self.bucket.objects.filter(Prefix=filename[:-1]):
				self.s3t.delete(self.BUCKET_NAME, obj.key)
			self.reset_buffer()
		else:
			self.s3t.delete(self.BUCKET_NAME, filename)

		return True

	def load_file(self,filename):
		return self.load_file_global(self.dataset+filename)

	def load_file_bytes(self,filename, b_i, b_f):
		return self.load_file_global_bytes(self.dataset+filename,b_i,b_f)

	def load_file_global(self,filename):
		return self.resource.meta.client.get_object(Bucket=self.BUCKET_NAME,Key=filename)['Body']

	def load_file_global_bytes(self,filename,b_i,b_f):
		bytes_rage = 'bytes=' + str(bi) + '-' + str(b_f)
		return self.resource.meta.client.get_object(Bucket=self.BUCKET_NAME,Key=filename,Range=bytes_rage)['Body']

	def load_file_metadata(self,filename,debug=False):
		# gets the metadata
		obj = self.resource.Object(self.BUCKET_NAME,self.dataset+filename).get()

		metadata = {
			'key' : self.dataset+filename,
			'date_loaded' : obj['ResponseMetadata']['HTTPHeaders']['date'],
			'date_added' : obj['ResponseMetadata']['HTTPHeaders']['date'],
			'last_modified' : obj['ResponseMetadata']['HTTPHeaders']['last-modified'],
			'HostId' : obj['ResponseMetadata']['HostId'],
		}

		return metadata

	def load_file_metadata_global(self,filename,debug=False):
		# gets the metadata
		obj = self.resource.Object(self.BUCKET_NAME,filename).get()

		metadata = {
			'key' : self.dataset+filename,
			'date_loaded' : obj['ResponseMetadata']['HTTPHeaders']['date'],
			'date_added' : obj['ResponseMetadata']['HTTPHeaders']['date'],
			'last_modified' : obj['ResponseMetadata']['HTTPHeaders']['last-modified'],
			'HostId' : obj['ResponseMetadata']['HostId'],
		}

		return metadata

	def list_files_in_path(self,path):
		for obj in self.bucket.objects.filter(Prefix=path):
			print(f'-- {obj.key}')
		return True

	def load_dataset(self):
		# loads all the metadata associated with the dataset
		files = []
		params = {"Bucket" : self.BUCKET_NAME, "Prefix" : self.dataset}
		objects = self.resource.meta.client.list_objects_v2(**params)

		# labels each file
		if 'Contents' in objects.keys():
			for obj in objects['Contents']:
				if obj["Key"][-1] != '/':
					metadata = {
						'key' : obj['Key'],
						'last_modified' : obj["LastModified"].strftime("%m/%d/%Y, %H:%M:%S"),
					}
					files.append(metadata)
			return files
		else:
			return []

	def load_dataset_list(self):
		# loads all the metadata associated with the dataset
		return self.load_list_in_path(self.dataset)

	def load_list_in_path(self,path):
		# loads all the metadata associated within a path
		keys = []
		last_m = []

		params = {"Bucket" : self.BUCKET_NAME, "Prefix" : path}
		objects = self.resource.meta.client.list_objects_v2(**params)

		# loads each file
		if 'Contents' in objects.keys():
			for obj in objects['Contents']:
				if obj['Key'][-1] != '/':
					keys.append(obj['Key'])
					last_m.append(obj["LastModified"].strftime("%m/%d/%Y, %H:%M:%S"))

			return keys, last_m
		else:
			return [],[]

	def check_if_empty(self,path):
		# checks the number of files in a directory
		summary = self.bucket.objects.filter(Prefix=path)
		count = 0
		for obj in summary:
			if obj.key[-1] != '/':
				count += 1
		return (count == 0)

	def load_files_in_dataset(self):
		# lists all the files in the main path of the dataset
		return self.list_files_in_path(self.dataset)

	def copy_file(self,filepath,full_target_name):
		# adding new file
		return self.copy_file_global(self.dataset+filepath,self.dataset+full_target_name)

	def copy_file_global(self,filepath,full_target_name):
		# adding new file
		copy_source = {
			'Bucket' : self.BUCKET_NAME,
			'Key' : filepath
		}
		self.s3t.copy(copy_source=copy_source,bucket=self.BUCKET_NAME, key=full_target_name)
		return True

	def get_size_of_file(self, filepath):
		return self.get_size_of_file_global(self.dataset + filepath)

	def get_size_of_file_global(self, filepath):
		return self.bucket.Object(filepath).content_length

	def reset_buffer(self):
		if self.s3t != None:
			self.s3t.shutdown()
		botocore_config = botocore.config.Config(max_pool_connections=80)
		self.resource.meta.client.config = botocore_config
		transfer_config = s3transfer.TransferConfig(use_threads=True,max_concurrency=80,multipart_threshold=25*1024,multipart_chunksize=25*1024)
		self.s3t = s3transfer.create_transfer_manager(self.resource.meta.client, transfer_config)

def main():
	# connects a bucket
	cloud = S3Bucket('stacktest123')
	cloud.connect_bucket()

	# connects to a dataset in the bucket
	cloud.create_dataset('dataset1/')

	# list the files in the bucket
	cloud.list_files_in_path('')
	cloud.load_files_in_dataset()

	# addes on file
	cloud.add_file('../../tests/image.png','image.png')
	cloud.add_file('../../tests/image.png','base/image.png',subpath='base/')
	cloud.reset_buffer()

	# opens one file
	body = cloud.load_file('image.png')
	print(body)

	meta = cloud.load_file_metadata('image.png')
	print(meta)

	# adds two file from binaries
	cloud.add_file_from_binary('extra/image2.png',open("../../tests/image.png", "rb"))
	cloud.add_file_from_binary('extra/image3.png',cloud.load_file('base/image.png'))
	cloud.reset_buffer()

	# removes a file
	cloud.remove_file('extra/image2.png')
	cloud.reset_buffer()

	# loads the dataset in memory
	dataset = cloud.load_dataset()
	print(cloud.load_dataset())

	cloud.load_files_in_dataset()

	cloud.copy_file('extra/image3.png','extra/image4.png')
	cloud.list_files_in_path('')
	print('success')

if __name__ == '__main__':
	main()



import sys
sys.path.append( '../../' )
import socket
import string
import json
import io
from datetime import datetime
import time
from pathlib import Path
import os
from src.comm.docker_ver import *
path_home = os.getenv('LCP_DKR')+'/' if docker_ver() else str(Path.home())

class projects:
    def __init__(self, project, init):
        self.init = init
        self.project = project
        self.projects = {}
        self.predictions = {}
        self.prefix_projects = init.prefix_meta + 'projects/' 
        self.prefix_experiments = self.prefix_projects + project + '/experiments/' 
        self.prefix_models = self.prefix_projects + project + '/models/'
        self.prefix_predictions = self.prefix_projects + project + '/predictions/'

        self.experiments = None
        self.models = None
        self.in_log = None
        self.logs = None
        self.current_run: int = None
        
    def verify_setup(self):
        if self.init.storage.check_if_empty(self.prefix_projects):
            return False
        if self.init.storage.check_if_empty(self.prefix_experiments):
            return False
        return True

    def init_project(self):
        if not self.verify_setup():
            self.setup_experiments()
            self.setup_models()
        self.experiments = json.load(self.init.storage.load_file_global(self.prefix_experiments + 'experiments.json'))
        self.models = json.load(self.init.storage.load_file_global(self.prefix_models + 'models.json'))
        try: 
            self.projects = json.load(self.init.storage.load_file_global(self.prefix_projects + 'projects.json'))
        except:
            self.projects = {}
        if not self.project in self.projects.keys():
            self.projects[self.project] = {'date created': datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), 'runs': [], 'models': []}
            self.init.storage.add_file_from_binary_global(self.prefix_projects + 'projects.json',io.BytesIO(json.dumps(self.projects).encode('ascii')))
        self.logs = None
        return True

    def setup_experiments(self):
        # creates json for experiments in project
        # List<dict>:
        #   dates: strings (dates of each run)
        #   logs: strings (URIS of the training logs of each run)
        #   models: strings (URIS of the models of each run)
        #   dataset_versions: strings (URIS of the dataset versions file for each run)
        experiments = []
        self.experiments = experiments
        json_path = self.prefix_experiments + 'experiments.json'

        # stores json in .stack folder
        self.init.storage.add_file_from_binary_global(json_path,io.BytesIO(json.dumps(experiments).encode('ascii')))
        self.init.storage.reset_buffer()
        return True

    def setup_models(self):
        # creates json for models in project
        models = {'date': [], 'uri': []}
        self.models = models
        json_path = self.prefix_models + 'models.json'
        
        self.init.storage.add_file_from_binary_global(json_path,io.BytesIO(json.dumps(models).encode('ascii')))
        self.init.storage.reset_buffer()
        return True

    def init_run(self):
        # checks run number
        if len(self.experiments) == 0:
            self.current_run = 0
            print('NNNNNN')
            print(self.current_run)
        else:
            self.current_run = self.get_latest_run_number()
            print('ADDDDD')
            print(self.current_run)
        
        self.logs = []

        # setups up empty run
        run = {'date': datetime.now().strftime("%m/%d/%Y, %H:%M:%S"), 'logs': self.prefix_experiments + 'logs/' + str(self.current_run).zfill(10), 'models': [], 'dataset_version': ''}
        self.experiments.append(run)
        self.projects[self.project]['runs'].append(run)
        self.init.storage.add_file_from_binary_global(self.prefix_projects + 'projects.json',io.BytesIO(json.dumps(self.projects).encode('ascii')))
        return True

    def add_prediction(self, data, label = '', version = None, model = None):
        if label is None:
            label = datetime.now().strftime("%m/%d/%Y, %H:%M:%S")
        pred_path = self.prefix_predictions+'/predictions/'+str(self.get_latest_prediction_number() + 1).zfill(10)
        self.predictions[datetime.now().strftime("%m/%d/%Y, %H:%M:%S")] = {'label': label, 'version': version, 'model': model, 'path': pred_path}
        
        self.init.storage.add_file_from_binary_global(self.prefix_predictions + 'predictions.json',io.BytesIO(json.dumps(self.predictions).encode('ascii')))
        self.init.storage.reset_buffer()
        
        self.init.storage.add_file_from_binary_global(pred_path,io.BytesIO(json.dumps(data).encode('ascii')))
        self.init.storage.reset_buffer()

    def add_log(self, data):
        if self.logs is None:
            self.logs = []
            self.init_run()

        self.logs.append(data)
        self.init.storage.add_file_from_binary_global(self.experiments[-1]['logs'],io.BytesIO(json.dumps(self.logs).encode('ascii')))
        self.init.storage.reset_buffer()
        
        json_path = self.prefix_experiments + 'experiments.json'
        self.init.storage.add_file_from_binary_global(json_path,io.BytesIO(json.dumps(self.experiments).encode('ascii')))
        self.init.storage.reset_buffer()

    def add_model(self, model, label = None):
        if label is None:
            label = 'model'
        if self.current_run is None:
            self.current_run = self.get_latest_run_number()

        model_path = self.prefix_models + '/' + str(self.current_run).zfill(10) + '/' + label
        
        self.experiments[-1]['models'].append(model_path)
        self.models['uri'].append(model_path)
        self.models['date'].append(datetime.now().strftime("%m/%d/%Y, %H:%M:%S"))

        self.projects[self.project]['models'].append(model_path)
        self.init.storage.add_file_from_binary_global(self.prefix_projects + 'projects.json',io.BytesIO(json.dumps(self.projects).encode('ascii')))

        self.init.storage.add_file_from_binary_global(model_path,model)
        self.init.storage.reset_buffer()
        
        json_path = self.prefix_models + 'models.json'
        self.init.storage.add_file_from_binary_global(json_path,io.BytesIO(json.dumps(self.models).encode('ascii')))
        self.init.storage.reset_buffer()
        
        json_path = self.prefix_experiments + 'experiments.json'
        self.init.storage.add_file_from_binary_global(json_path,io.BytesIO(json.dumps(self.experiments).encode('ascii')))
        self.init.storage.reset_buffer()

    def get_model(self, label):
        model_path = ''
        for model_path in self.models['uri']:
            if label in model_path:
                return self.init.storage.load_file_global(model_path)
        return None
    
    def get_latest_run_number(self):
		# checks all the runs
        run_path, _ = self.init.storage.load_list_in_path(self.prefix_experiments + 'logs/')

		# gets the list in number
        run_path = [int(x.replace(self.prefix_experiments+'logs/','')) for x in run_path]

        if len(run_path):
            return max(run_path) + 1
        else:
            return 0

    def get_latest_prediction_number(self):
		# checks all the prediction
        try:
            self.predictions = json.load(self.init.storage.load_file_global(self.prefix_predictions + 'predictions.json'))
            return len(self.predictions.keys())
        except:
            self.init.storage.add_file_from_binary_global(self.prefix_predictions + 'predictions.json',io.BytesIO(json.dumps({}).encode('ascii')))
            return 0


import sys
sys.path.append( '../../..' )
from src.user.database.database import MongoClient
from src.user.user import User
import random
import string
import hashlib
from datetime import date, timedelta
import json 
from pathlib import Path

class License(object):
	"""docstring for License"""
	def __init__(self, ):
		super(License, self).__init__()
	
	def generateLicense(self, user, days_lic):
		# generates license
		exp_date = date.today()+timedelta(days=days_lic)
		lic = [random.choice(string.ascii_letters) for i in range(15)]
		license = ""
		for ele in lic: 
			license += ele

		dic_cloud = {
			'username' : user.username,
			'exp_day' : exp_date.day,
			'exp_month' : exp_date.month,
			'exp_year' : exp_date.year,
			'license'  : license
		}

		mongo_client = MongoClient()
		lic_db = mongo_client.getDatabase('lic_db')
		lic_col = mongo_client.getCollection('lic_data',lic_db)

		mongo_client.insert2Collection(dic_cloud,lic_col)

		dic_local = {
			'user' : user.username,
			'license'  : hashlib.sha256(license.encode('utf-8')).hexdigest()
		}

		json_object = json.dumps(dic_local)
		with open(str(Path.home())+"/stack_license.lic", "w") as outfile:
			outfile.write(json_object)

		return True

	def checkLicense(self, user):
		mongo_client = MongoClient()
		lic_db = mongo_client.getDatabase('lic_db')
		lic_col = mongo_client.getCollection('lic_data',lic_db)

		# checks user does not exist yet
		user_id = mongo_client.findInCollection(lic_col,{'username' : user.username})

		l = 0
		for x in user_id:
			l = l + 1
			ref = hashlib.sha256(x['license'].encode('utf-8')).hexdigest()
			lic = x

		f = open(str(Path.home())+"/stack_license.lic")
		data = json.load(f)
		f.close()

		# checks if the license is valide
		valid = True

		if date.today().year > lic['exp_year']:
			valid = False
		elif date.today().year == lic['exp_year']:
			if date.today().month > lic['exp_month']:
				valid = False
			elif date.today().month == lic['exp_month']:
					if date.today().day > lic['exp_day']:
						valid = False
		
		if not valid:
			print('license expired...')
			return False

		if ref != data['license']:
			print('invalid license')
			return False

		print('valid stack_license')
		return True

def main():
	usr = User()
	usr.loginUser()

	lic = License()
	lic.generateLicense(usr,100)
	lic.checkLicense(usr)

	usr.deleteUser()

if __name__ == '__main__':
	main()